@@ -24,6 +24,10 @@ import logging
 import os
 import shutil
 import subprocess
+import tempfile
+
+from . import vmdb2
+from . import vmdebootstrap
 
 BASE_PACKAGES = [
     'initramfs-tools',
@@ -38,6 +42,7 @@ class ImageBuilder(object):  # pylint: disable=too-many-instance-attributes
     machine = 'all'
     free = True
 
+    builder_backend = 'vmdebootstrap'
     root_filesystem_type = 'btrfs'
     boot_filesystem_type = None
     boot_size = None
@@ -68,10 +73,13 @@ class ImageBuilder(object):  # pylint: disable=too-many-instance-attributes
         """Initialize object."""
         self.arguments = arguments
         self.packages = BASE_PACKAGES
-        self.parameters = []
-        self.environment = []
 
-        self.files_to_clean = []
+        self.ram_directory = None
+
+        self.builder_backends = {}
+        self.builder_backends['vmdebootstrap'] = \
+            vmdebootstrap.VmdebootstrapBuilderBackend(self)
+        self.builder_backends['vmdb2'] = vmdb2.Vmdb2BuilderBackend(self)
 
         self.image_file = os.path.join(
             self.arguments.build_dir, self._get_image_base_name() + '.img')
@@ -90,6 +98,12 @@ class ImageBuilder(object):  # pylint: disable=too-many-instance-attributes
 
     def cleanup(self):
         """Finalize tasks."""
+        logger.info('Cleaning up')
+        if self.ram_directory:
+            self._run(['sudo', 'umount', self.ram_directory.name])
+            self.ram_directory.cleanup()
+            self.ram_directory = None
+
         logger.removeHandler(self.log_handler)
 
     def build(self):
@@ -106,6 +120,10 @@ class ImageBuilder(object):  # pylint: disable=too-many-instance-attributes
 
         self.sign(archive_file)
 
+    def make_image(self):
+        """Call a builder backend to create basic image."""
+        self.builder_backends[self.builder_backend].make_image()
+
     def _get_image_base_name(self):
         """Return the base file name of the final image."""
         free_tag = 'free' if self.free else 'nonfree'
@@ -116,127 +134,25 @@ class ImageBuilder(object):  # pylint: disable=too-many-instance-attributes
                 build_stamp=self.arguments.build_stamp, machine=self.machine,
                 architecture=self.architecture)
 
-    def make_image(self):
-        """Create a disk image."""
-        if self.should_skip_step(self.image_file):
-            logger.info('Image exists, skipping build - %s', self.image_file)
-            return
+    def get_temp_image_file(self):
+        """Get the temporary path to where the image should be built.
 
-        temp_image_file = self.image_file + '.temp'
-        self.execution_wrapper = ['sudo', '-H']
-        self.parameters = [
-            '--hostname', self.arguments.hostname,
-            '--image', temp_image_file,
-            '--size', self.arguments.image_size,
-            '--mirror', self.arguments.build_mirror,
-            '--distribution', self.arguments.distribution,
-            '--arch', self.architecture,
-            '--lock-root-password',
-            '--log', self.log_file,
-            '--log-level', self.arguments.log_level,
-            '--verbose',
-            '--customize', self.customization_script,
-        ]
-        self.environment = {
-            'MIRROR': self.arguments.mirror,
-            'BUILD_MIRROR': self.arguments.build_mirror,
-            'MACHINE': self.machine,
-            'SOURCE': 'true' if self.arguments.include_source else 'false',
-            'SUITE': self.arguments.distribution,
-            'ENABLE_NONFREE': 'no' if self.free else 'yes',
-        }
-        self.process_variant()
-        self.process_architecture()
-        self.process_boot_loader()
-        self.process_kernel_flavor()
-        self.process_filesystems()
-        self.process_packages()
-        self.process_custom_packages()
-        self.process_environment()
+        If building to RAM is enabled, create a temporary directory, mount
+        tmpfs in it and return a path in that directory. This is so that builds
+        that happen in RAM will be faster.
 
-        command = self.execution_wrapper + [self.arguments.vmdebootstrap] + \
-            self.parameters
-        self._run(command)
+        If building to RAM is disabled, append .temp to the final file name and
+        return it.
 
-        os.rename(temp_image_file, self.image_file)
+        """
+        if not self.arguments.build_in_ram:
+            return self.image_file + '.temp'
 
-    def process_variant(self):
-        """Add paramaters for deboostrap variant."""
-        if self.debootstrap_variant:
-            self.parameters += \
-                ['--debootstrapopts', 'variant=' + self.debootstrap_variant]
-
-    def process_architecture(self):
-        """Add parameters specific to the architecture."""
-        if self.architecture not in ('i386', 'amd64'):
-            self.parameters += ['--foreign', '/usr/bin/qemu-arm-static']
-
-            # Using taskset to pin build process to single core. This
-            # is a workaround for a qemu-user-static issue that causes
-            # builds to hang. (See Debian bug #769983 for details.)
-            self.execution_wrapper = \
-                ['taskset', '0x01'] + self.execution_wrapper
-
-    def process_boot_loader(self):
-        """Add parameters related to boot loader."""
-        option_map = {
-            'grub': ['--grub'],
-            'extlinux': [],
-            'u-boot': ['--no-extlinux'],
-            None: ['--no-extlinux']
-        }
-        self.parameters += option_map[self.boot_loader]
-
-        if self.boot_loader == 'u-boot':
-            self.parameters += [
-                '--package', 'u-boot-tools', '--package', 'u-boot']
-
-        if self.boot_size:
-            self.parameters += ['--bootsize', self.boot_size]
-
-        if self.boot_offset:
-            self.parameters += ['--bootoffset', self.boot_offset]
-
-    def process_kernel_flavor(self):
-        """Add parameters for kernel flavor."""
-        if self.kernel_flavor == 'default':
-            return
-
-        if self.kernel_flavor is None:
-            self.parameters += ['--no-kernel']
-            return
-
-        self.parameters += ['--kernel-package',
-                            'linux-image-' + self.kernel_flavor]
-
-    def process_filesystems(self):
-        """Add parameters necessary for file systems."""
-        self.parameters += ['--roottype', self.root_filesystem_type]
-        if self.boot_filesystem_type:
-            self.parameters += ['--boottype', self.boot_filesystem_type]
-
-        if 'btrfs' in [self.root_filesystem_type, self.boot_filesystem_type]:
-            self.packages += ['btrfs-tools']
-
-    def process_packages(self):
-        """Add parameters for additional packages to install in image."""
-        for package in self.packages + (self.arguments.package or []):
-            self.parameters += ['--package', package]
-
-    def process_custom_packages(self):
-        """Add parameters for custom DEB packages to install in image."""
-        for package in (self.arguments.custom_package or []):
-            if 'plinth_' in package:
-                self.environment['CUSTOM_PLINTH'] = package
-            elif 'freedombox-setup_' in package:
-                self.environment['CUSTOM_SETUP'] = package
-            else:
-                self.parameters += ['--custom-package', package]
-
-    def process_environment(self):
-        """Add environment we wish to pass to the command wrapper: sudo."""
-        for key, value in self.environment.items():
-            self.execution_wrapper += [key + '=' + value]
+        self.ram_directory = tempfile.TemporaryDirectory()
+        self._run(
+            ['sudo', 'mount', '-t', 'tmpfs', 'tmpfs', self.ram_directory.name])
+        return os.path.join(self.ram_directory.name,
+                            os.path.basename(self.image_file))
 
     def compress(self, archive_file, image_file):
         """Compress the generate image."""
@@ -317,6 +233,7 @@ class AMD64ImageBuilder(AMDIntelImageBuilder):
 class I386ImageBuilder(AMDIntelImageBuilder):
     """Image builder for all i386 targets."""
     architecture = 'i386'
+    kernel_flavor = '686'
 
 
 class VMImageBuilder(AMDIntelImageBuilder):
@@ -386,6 +303,7 @@ class VirtualBoxAmd64ImageBuilder(VirtualBoxImageBuilder):
 class VirtualBoxI386ImageBuilder(VirtualBoxImageBuilder):
     """Image builder for all VirutalBox i386 targets."""
     architecture = 'i386'
+    kernel_flavor = '686'
 
 
 class VagrantImageBuilder(VirtualBoxAmd64ImageBuilder):
@@ -480,6 +398,7 @@ class QemuAmd64ImageBuilder(QemuImageBuilder):
 class QemuI386ImageBuilder(QemuImageBuilder):
     """Image builder for all Qemu i386 targets."""
     architecture = 'i386'
+    kernel_flavor = '686'
 
 
 class ARMImageBuilder(ImageBuilder):
@@ -524,6 +443,11 @@ class A20OLinuXinoMicroImageBuilder(A20ImageBuilder):
     machine = 'a20-olinuxino-micro'
 
 
+class BananaProImageBuilder(A20ImageBuilder):
+    """Image builder for Banana Pro target."""
+    machine = 'banana-pro'
+
+
 class Cubieboard2ImageBuilder(A20ImageBuilder):
     """Image builder for Cubieboard 2 target."""
     machine = 'cubieboard2'
@@ -543,7 +467,7 @@ class DreamPlugImageBuilder(ARMImageBuilder):
     """Image builder for DreamPlug target."""
     architecture = 'armel'
     machine = 'dreamplug'
-    kernel_flavor = 'kirkwood'
+    kernel_flavor = 'marvell'
     boot_filesystem_type = 'vfat'
 
 
@@ -565,3 +489,12 @@ class RaspberryPi2ImageBuilder(ARMImageBuilder):
     free = False
     boot_offset = '64mib'
     kernel_flavor = 'armmp'
+
+
+class RaspberryPi3ImageBuilder(ARMImageBuilder):
+    """Image builder for Raspberry Pi 3 target."""
+    architecture = 'armhf'
+    machine = 'raspberry3'
+    free = False
+    boot_offset = '64mib'
+    kernel_flavor = 'armmp'
