@@ -47,11 +47,13 @@ ARCHITECTURES = {
     'a20-olinuxino-lime': 'armhf',
     'a20-olinuxino-lime2': 'armhf',
     'a20-olinuxino-micro': 'armhf',
+    'banana-pro': 'armhf',
     'cubieboard2': 'armhf',
     'cubietruck': 'armhf',
     'dreamplug': 'armel',
     'raspberry': 'armel',
     'raspberry2': 'armhf',
+    'raspberry3': 'armhf',
 }
 
 
@@ -118,17 +120,19 @@ class TestInvocation(unittest.TestCase):
             'a20-olinuxino-lime': 'a20-olinuxino-lime-armhf.img',
             'a20-olinuxino-lime2': 'a20-olinuxino-lime2-armhf.img',
             'a20-olinuxino-micro': 'a20-olinuxino-micro-armhf.img',
+            'banana-pro': 'banana-pro-armhf.img',
             'cubieboard2': 'cubieboard2-armhf.img',
             'cubietruck': 'cubietruck-armhf.img',
             'dreamplug': 'dreamplug-armel.img',
             'raspberry': 'raspberry-armel.img',
             'raspberry2': 'raspberry2-armhf.img',
+            'raspberry3': 'raspberry3-armhf.img',
         }
 
         distribution = distribution or 'unstable'
 
         free_tag = 'free'
-        if target in ('raspberry', 'raspberry2'):
+        if target in ('raspberry', 'raspberry2', 'raspberry3'):
             free_tag = 'nonfree'
 
         file_name = 'freedombox-{distribution}-{free_tag}_{build_stamp}_' \
@@ -237,14 +241,22 @@ class TestInvocation(unittest.TestCase):
         self.assert_environment_passed({'SUITE': distribution},
                                        distribution=distribution)
 
-    def test_include_source(self):
-        """Test that include-source parameter works."""
+    def test_download_source(self):
+        """Test that download-source parameter works."""
         self.invoke()
         self.assert_environment_passed({'SOURCE': 'false'})
 
-        self.invoke(include_source=True, force=True)
+        self.invoke(download_source=True, force=True)
         self.assert_environment_passed({'SOURCE': 'true'})
 
+    def test_include_source(self):
+        """Test that include-source parameter works."""
+        self.invoke()
+        self.assert_environment_passed({'SOURCE_IN_IMAGE': 'false'})
+
+        self.invoke(include_source=True, force=True)
+        self.assert_environment_passed({'SOURCE_IN_IMAGE': 'true'})
+
     def test_package(self):
         """Test that package parameter works."""
         package = self.random_string()
@@ -371,11 +383,16 @@ class TestInvocation(unittest.TestCase):
                 self.assert_arguments_not_passed(['--package', 'u-boot'])
                 self.assert_arguments_not_passed(['--package', 'u-boot-tools'])
                 self.assert_arguments_not_passed(['--no-extlinux'])
-            elif target in ('raspberry', 'raspberry2'):
+            elif target in ('raspberry'):
                 self.assert_arguments_not_passed(['--grub'])
                 self.assert_arguments_not_passed(['--package', 'u-boot'])
                 self.assert_arguments_not_passed(['--package', 'u-boot-tools'])
                 self.assert_arguments_passed(['--no-extlinux'])
+            elif target in ('raspberry2', 'raspberry3'):
+                self.assert_arguments_not_passed(['--grub'])
+                self.assert_arguments_passed(['--package', 'u-boot'])
+                self.assert_arguments_passed(['--package', 'u-boot-tools'])
+                self.assert_arguments_passed(['--no-extlinux'])
             else:
                 self.assert_arguments_not_passed(['--grub'])
                 self.assert_arguments_passed(['--package', 'u-boot'])
@@ -394,22 +411,27 @@ class TestInvocation(unittest.TestCase):
             if architecture in ('i386', 'amd64'):
                 self.assert_arguments_passed(['--roottype', 'btrfs'])
                 self.assert_arguments_not_passed(['--boottype'])
-                self.assert_arguments_passed(['--package', 'btrfs-tools'])
+                self.assert_arguments_passed(['--package', 'btrfs-progs'])
                 self.assert_arguments_not_passed(['--bootsize', '128M'])
-            elif target in ('raspberry', 'raspberry2'):
+            elif target in ('raspberry'):
                 self.assert_arguments_passed(['--roottype', 'ext4'])
                 self.assert_arguments_passed(['--boottype', 'vfat'])
-                self.assert_arguments_not_passed(['--package', 'btrfs-tools'])
+                self.assert_arguments_not_passed(['--package', 'btrfs-progs'])
+                self.assert_arguments_passed(['--bootsize', '128M'])
+            elif target in ('raspberry2', 'raspberry3'):
+                self.assert_arguments_passed(['--roottype', 'btrfs'])
+                self.assert_arguments_passed(['--boottype', 'ext2'])
+                self.assert_arguments_passed(['--package', 'btrfs-progs'])
                 self.assert_arguments_passed(['--bootsize', '128M'])
             elif target in ('dreamplug'):
                 self.assert_arguments_passed(['--roottype', 'btrfs'])
                 self.assert_arguments_passed(['--boottype', 'vfat'])
-                self.assert_arguments_passed(['--package', 'btrfs-tools'])
+                self.assert_arguments_passed(['--package', 'btrfs-progs'])
                 self.assert_arguments_passed(['--bootsize', '128M'])
             else:
                 self.assert_arguments_passed(['--roottype', 'btrfs'])
                 self.assert_arguments_passed(['--boottype', 'ext2'])
-                self.assert_arguments_passed(['--package', 'btrfs-tools'])
+                self.assert_arguments_passed(['--package', 'btrfs-progs'])
                 self.assert_arguments_passed(['--bootsize', '128M'])
 
     def test_boot_offset(self):
@@ -417,9 +439,11 @@ class TestInvocation(unittest.TestCase):
         for target, architecture in ARCHITECTURES.items():
             self.build_stamp = self.random_string()
             self.invoke([target])
-            if target in ('raspberry', 'raspberry2', 'dreamplug') or \
+            if target in ('raspberry', 'dreamplug') or \
                architecture in ('i386', 'amd64'):
                 self.assert_arguments_not_passed(['--bootoffset'])
+            elif target in ('raspberry2', 'raspberry3'):
+                self.assert_arguments_passed(['--bootoffset', '64mib'])
             elif target in ('beaglebone'):
                 self.assert_arguments_passed(['--bootoffset', '2mib'])
             else:
@@ -430,20 +454,27 @@ class TestInvocation(unittest.TestCase):
         for target, architecture in ARCHITECTURES.items():
             self.build_stamp = self.random_string()
             self.invoke([target])
-            if architecture in ('i386', 'amd64'):
+            if architecture in ('amd64'):
                 self.assert_arguments_not_passed(['--no-kernel'])
                 self.assert_arguments_not_passed(['--kernel-package'])
+            elif architecture in ('i386'):
+                self.assert_arguments_not_passed(['--no-kernel'])
+                self.assert_arguments_passed(
+                    ['--kernel-package', 'linux-image-686'])
             elif target in ('beaglebone'):
                 self.assert_arguments_not_passed(['--no-kernel'])
                 self.assert_arguments_passed(
                     ['--kernel-package', 'linux-image-armmp'])
-            elif target in ('raspberry', 'raspberry2'):
+            elif target in ('raspberry'):
                 self.assert_arguments_passed(['--no-kernel'])
                 self.assert_arguments_not_passed(['--kernel-package'])
+            elif target in ('raspberry2', 'raspberry3'):
+                self.assert_arguments_passed(
+                    ['--kernel-package', 'linux-image-armmp'])
             elif target in ('dreamplug'):
                 self.assert_arguments_not_passed(['--no-kernel'])
                 self.assert_arguments_passed(
-                    ['--kernel-package', 'linux-image-kirkwood'])
+                    ['--kernel-package', 'linux-image-marvell'])
             else:
                 self.assert_arguments_not_passed(['--no-kernel'])
                 self.assert_arguments_passed(
